/**
 *  Copyright 2009-2011 Matus Chochlik. Distributed under the Boost
 *  Software License, Version 1.0. (See accompanying file
 *  LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 *
 *  WARNING: this file is here for development / testing purposes only
 *  and its contents and input / output can change witout any prior
 *  notice.
 */

#include <mirror/mirror.hpp>
#include <mirror/by_name.hpp>
#include <puddle/puddle.hpp>

#include <mirror/meta_prog/accumulate.hpp>


#include <cassert>
#include <iostream>
#include <stdexcept>

#include "./test.hpp"

double g_x = 123.456;

namespace test {

double g_x = 234.567;
person g_p = {person::salutation::mr, "Johnny", "B.", "Goode", std::tm(), gender::male, 170.f, 80.f};

}

MIRROR_NAMESPACE_BEGIN

MIRROR_REG_GLOBAL_SCOPE_VARIABLE(_, g_x)
MIRROR_REG_FREE_VARIABLE(_, test, g_x)
MIRROR_REG_FREE_VARIABLE(_, test, g_p)

MIRROR_NAMESPACE_END

PUDDLE_NAMESPACE_BEGIN
namespace aux {


} // namespace aux

PUDDLE_NAMESPACE_END

struct name_printer
{
	template <typename MetaObject>
	void operator()(MetaObject mo, bool first, bool last) const
	{
		std::cout << mo.base_name() << std::endl;
	}
};

struct json_printer
{
	template <typename T>
	static std::true_type is_writable(T*, decltype(std::cout << (*((T*)nullptr)) == std::cout) = false);
	static std::false_type is_writable(...);

	template <typename T>
	static auto writable(const T&) -> decltype(is_writable((T*)nullptr));

	template <typename MetaObject>
	void do_print(
		MetaObject mo,
		mirror::meta_type_tag,
		std::true_type
	) const
	{
		std::cout << "'" << mo.get() << "'";
	}

	template <typename MetaObject>
	void do_print(
		MetaObject mo,
		mirror::meta_enum_tag,
		std::false_type
	) const
	{
		std::cout << "'" << mo.value_name() << "'";
	}

	template <typename MetaObject>
	void do_print(
		MetaObject mo,
		mirror::meta_class_tag,
		std::false_type
	) const
	{
		if(mo.attributes().empty()) std::cout << "N/A";
		else
		{
			std::cout << "{";
			mo.attributes().for_each(*this);
			std::cout << "}";
		}
	}

	template <typename MetaObject>
	void operator()(MetaObject mo, bool first, bool last) const
	{
		std::cout << "'" << mo.variable().base_name() << "': ";
		decltype(writable(mo.get())) wr;
		auto cat = mo.variable().type().category();
		do_print(mo, cat, wr);
		if(!last) std::cout << ", ";
	}
};

struct concatenator
{
	template <typename MetaObject>
	std::string operator()(
		std::string value,
		MetaObject mo,
		bool first,
		bool last
	) const
	{
		value.append(mo.base_name());
		if(last) value.append(".");
		else value.append(", ");
		return value;
	}
};

int main(void)
{
	try
	{
		using namespace mirror;
		auto  mi = puddle::reflected_type<int>();
		auto  mp = puddle::reflected_type<test::person>();
		auto  mg = puddle::reflected_type<test::gender>();
		//
		assert(mp.is_a(meta_type_tag()));
		assert(mp.is_a(meta_class_tag()));
		assert(mp.is_type());
		assert(mp.base_name() == "person");
		assert(mp.scope().base_name() == "test");
		assert(mp.scope().scope().is_a(meta_global_scope_tag()));
		assert(!mp.scope().members().empty());
		assert(!mp.scope().scope().members().empty());
		assert(mp.full_name() == "test::person");
		assert(mp.elaborated_type().keyword() == "struct");
		assert(mp.elaborated_type().is_struct());
		assert(mp.members().at_c<6>().storage_class().is_none());
		assert(mp.members().at_c<6>().access_type().is_public());
		assert(mp.members().find(mp.members().at_c<2>()).size() == mp.members().step_front().step_front().size());
		//
		assert(mi.apply<std::is_integral>());
		assert(mp.apply<std::is_class>());
		assert(mg.apply<std::is_enum>());
		assert(mp.apply<std::is_class>() == mp.is_class());
		assert(mg.apply<std::is_enum>() == mg.is_enum());
		std::cout << mp.apply<std::add_pointer>().base_name() << std::endl;
		//
		typedef decltype(mp.mirrored()) meta_person;
		std::cout << meta_person::full_name() << std::endl;
		std::cout << mp.self().construct_name() << std::endl;
		std::cout << mp.scope().self().construct_name() << std::endl;
		mp.members().for_each(name_printer());
		mp.scope().members().for_each(name_printer());
		mp.scope().scope().members().for_each(name_printer());
		//
		//
		std::cout << mp.members().accumulate<std::string>(concatenator(), "") << std::endl;
		std::cout << mp.members().only_if_<
			std::is_enum<mirror::original_type<mirror::type_of<mirror::mp::arg<1> > > >
		>().accumulate<std::string>(concatenator(), "") << std::endl;
		//
		std::cout << mg.enum_values().front().full_name() << std::endl;
		std::cout << mg.enum_values().at_c<1>().full_name() << std::endl;
		std::cout << mg.enum_values().at_c<2>().full_name() << std::endl;
		//
		std::cout << mp->title().type().enum_values().accumulate<std::string>(concatenator()) << std::endl;
		std::cout << mp->sex().type().enum_values().accumulate<std::string>(concatenator()) << std::endl;
		//
		double x;
		typedef MIRRORED_SIMPLE_VARIABLE(_, x) meta_x;
		static_assert(
			std::is_same<
				mirror::meta_object_category<meta_x>::type,
				mirror::meta_variable_tag
			>::value,
			"wrong category for meta_x"
		);
		static_assert(
			std::is_same<
				mirror::meta_object_category<meta_x::scope>::type,
				mirror::meta_unspecified_scope_tag
			>::value,
			"wrong category for meta_x::scope"
		);
		std::cout << meta_x::full_name() << std::endl;
		std::cout << meta_x::scope::full_name() << std::endl;
		std::cout << meta_x::type::full_name() << std::endl;
		std::cout << MIRRORED_FREE_VARIABLE(test, g_x)::storage_class::keyword() << std::endl;
		std::cout << MIRRORED_GLOBAL_SCOPE_VARIABLE(g_x)::storage_class::keyword() << std::endl;
		std::cout << MIRRORED_FREE_VARIABLE(test, g_x)::get() << std::endl;
		MIRRORED_FREE_VARIABLE(test, g_x)::set(MIRRORED_GLOBAL_SCOPE_VARIABLE(g_x)::get());
		std::cout << MIRRORED_FREE_VARIABLE(test, g_x)::get() << std::endl;
		//
		auto meta_g_x = puddle::adapt<MIRRORED_FREE_VARIABLE(test, g_x)>();
		meta_g_x.set(456.789);
		std::cout << test::g_x << std::endl;
		//
		std::cout << meta_g_x.scope().free_variables().size() << std::endl;
		std::cout << meta_g_x.scope().free_variables_by_name().g_p().type().base_name() << std::endl;
		std::cout << meta_g_x.scope()->g_p().base_name() << std::endl;
		//
		auto meta_g_p_i = puddle::instance_of(meta_g_x.scope()->g_p());
		assert(meta_g_p_i.is_instance());
		assert(!meta_g_p_i.is_type());
		//
		std::cout << meta_g_p_i.attributes().size() << std::endl;
		meta_g_p_i.attributes().for_each(json_printer());
		std::cout << std::endl;
	}
	catch(std::exception const& error)
	{
		std::cerr << "Error: " << error.what() << std::endl;
	}
	//
	return 0;
}
